#ifndef __ACTIVYLCD_CPP
#define __ACTIVYLCD_CPP

#include "activylcd.h"

#undef LCDDEBUG

///////////////////////////////////////////////////
// activy300 LCD plugin for VDR
//
// Original written by:
//
// Meinrad Sauter <meinrad.sauter@gmx.de>
// andreas 'randy' weinberger <vdr@smue.org>
//
// rearranged, beautified and rewritten by:
//
// Markus Geisler <geisler@primusnetz.de>
//

/////////////////////////////////////////////////////////
// Konstruktor

activyLCD::activyLCD()
{
	char output[30];

#ifdef LCDDEBUG
	logfile = fopen("/tmp/debuglog.alcd","w");
#endif
	locked = 0;
	SerialPortOpen();
  	LCDInit();  

  	snprintf(output,2, "%s", HWREV);
  	SendCommandToLCD(output,2 );
  	snprintf(output,2, "%s", FWREV);
  	SendCommandToLCD(output,2 );

	char inittext1[21];
	char inittext2[21];
	snprintf(inittext1,21," FSC ACTIVY300 LCD  ");
	snprintf(inittext2,21," -- initalizing --  ");
	SetLCDLines(inittext1,inittext2);
	sleep(2);
	snprintf(inittext1,21,"   plugin for VDR   ");
	snprintf(inittext2,21,"  mg@primusnetz.de  ");
	SetLCDLines(inittext1,inittext2);
	sleep(2);
}

/////////////////////////////////////////////////////////
// Destruktor

activyLCD::~activyLCD()
{
  	LCDClear();
  	SetPowerButton(1);
  	SetLEDBrightness(LEDBrightnessOnExit);
  	SerialPortClose();
#ifdef LCDDEBUG
	fclose(logfile);
#endif
}

/////////////////////////////////////////////////////////
// Open Serial Port

int activyLCD::SerialPortOpen(void)
{
	struct termios options;

	// open serial device
  	fd = open(LCD_PORT, O_RDWR | O_NOCTTY | O_NDELAY);

	// check result
  	if (fd == -1) 
	{
		// error - port could not be opened
		return(-1);
  	} 
	else 
	{
		// ok - port opened
    		fcntl(fd, F_SETFL, 0);
	  	// Get the current options for the port...
		tcgetattr(fd, &options);
	  	// Set the baud rates to 38400...
	  	cfsetispeed(&options, B38400);
	  	cfsetospeed(&options, B38400);
	  	// Enable the receiver and set local mode...
	  	options.c_cflag |= (CLOCAL);
	  	options.c_cflag &= ~PARENB;
	  	options.c_cflag &= ~CSTOPB;
	  	options.c_cflag &= ~CSIZE;
	  	options.c_cflag |= CS8;
	  	options.c_oflag &= ~OPOST;
                options.c_oflag &= ~(ICANON | ECHO | ECHOE | ISIG);
                options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
		options.c_cc[VMIN]  = 0;                  /* warten auf min. 0 Zeichen */
		options.c_cc[VTIME] = 10;                 /* Timeout 10 Sekunden */

	  	// Set the new options for the port...
	  	tcsetattr(fd, TCSANOW, &options);

	  	return (fd);
  	}
}

/////////////////////////////////////////////////////////
// Close Serial Port

void activyLCD::SerialPortClose()
{
  	close(fd);
}

/////////////////////////////////////////////////////////
// Send Command to LCD-Device

void activyLCD::SendCommandToLCD(char *command,int length)
{
	char ergebnis1;
	char ergebnis2;
	char ergebnis3;
	char ergebnis4;
	char ergebnis5;
	char ergebnis6;
	int result;

	if(command && length && !locked)
	{
		locked = 1;

	  	write(fd,command,length);

                result = read(fd,&ergebnis1,1);
                result = read(fd,&ergebnis2,1);

		// ergebniswerte auswerten
		switch (ergebnis1)
		{
			// hw revision
			case '\x89' : 
			{
				// hw revision - 1 byte!
				if(ergebnis2 == '\xA0')
				{
	                		read(fd,&ergebnis3,1);
					isyslog("alcd-plugin: Activy300-LCD-HW Revision: %i\n",ergebnis3);
				}
				else
				{
					isyslog("alcd-plugin: Activy300-LCD-HW Revision could not be determined\n");
				}
				break;
			}
			// fw revision
			case '\x8a' : 
			{
				// fw revision - 4 bytes!
				if(ergebnis2 == '\xA0')
				{
	                		read(fd,&ergebnis3,1);
			                read(fd,&ergebnis4,1);
			                read(fd,&ergebnis5,1);
			                read(fd,&ergebnis6,1);

					isyslog("alcd-plugin: Activy300-LCD-SW Revision: Release: %i Version: %i PreRelease: %c VirtualNumber: %i\n",ergebnis3,ergebnis4,ergebnis5,ergebnis6);
				}
				else
				{
					isyslog("alcd-plugin: Activy300-LCD-SW Revision could not be determined\n");
				}
				break;
			}
			// set powerbutton
			case '\x94' : 
			{
				// set powerbutton
				if(ergebnis2 == '\xA0')
				{
					// powerbutton could not be set!
				}
				else
				{
					isyslog("alcd-plugin: Error during powerbutton set\n");
				}
				break;
			}
			// clear lcd
			case '\x9b' : 
			{
				// clear lcd
				if(ergebnis2 == '\xA0')
				{
					// lcd has been cleaned!
				}
				else
				{
					isyslog("alcd-plugin: Error during lcd clear\n");
				}
				break;
			}
			// writeline
			case '\x9a' : 
			{
				// writeline
				if(ergebnis2 == '\xA0')
				{
					// line has been written!
				}
				else
				{
					isyslog("alcd-plugin: Error during lcd clear\n");
				}
				break;
			}
			// set brightness
			case '\xf0' : 
			{
				// set brightness
				if(ergebnis2 == '\xA0')
				{
					// brightness could be set!
				}
				else
				{
					isyslog("alcd-plugin: Error during set brightness\n");
				}
				break;
			}
			default :
			{
				// command unknown!
				isyslog("alcd-plugin: Error: unknown resultcode \'%s\'%x\'!\n",command,ergebnis1);
			}
		}

		if(ergebnis2 != '\xA0')
		{
	                isyslog("alcd-plugin: ERROR: Ergebniswert: |%x|%c|%x|%c|\n",ergebnis1,ergebnis1,ergebnis2,ergebnis2);
			isyslog("alcd-plugin: ERROR: Performing LCD CLEAR!\n");
	  		write(fd,LCD_CLEAR,2);
                	result = read(fd,&ergebnis3,1);
                	result = read(fd,&ergebnis4,1);
			if(ergebnis4 != '\xA0')
			{
		                isyslog("alcd-plugin: ERROR: Ergebniswert LCD CLEAR: |%x|%c|\n",ergebnis4,ergebnis4);
			}
			usleep(100);
		}

		usleep(5000);

		locked = 0;
	}
}

/////////////////////////////////////////////////////////
// Init LCD Device

void activyLCD::LCDInit()
{
  	if(PowerButton)
 	{
  		SetPowerButton(1);
  	} 
	else 
	{
    		SetPowerButton(0);
  	}
  	SetLEDBrightness(LEDBrightness);
}

/////////////////////////////////////////////////////////
// Replace German Umlauts

char *activyLCD::ReplaceUmlaute(const char *line)
{
	// only proceed, if valid data
	if(line)
	{
	  	int b;  
	  	char c;
	  	char replacetemp1[strlen(line)*2];
		char *linetemp;
	        char replaced[strlen(line)*2];
	
		int size = strlen(line);
	
	  	replacetemp1[0]='\0'; 
	  	memset(replaced,' ',strlen(line)*2);
	
		linetemp = strdup(line);
	
	  	for (b=0; b<size; b++) 
		{
	  	  	c = linetemp[b];
	
	   	 	switch(c) 
			{
	     	 		case '\x09':
	      	  		strcat(replacetemp1," ");
	        		break; 
	
	    	 		case '':
	      	  		strcat(replacetemp1,"ae");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"oe");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"ue");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"ss");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Ae");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Oe");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Ue");
	        		break;
	
	      			default:
	        		replacetemp1[strlen(replacetemp1) + 1]=0;
	        		replacetemp1[strlen(replacetemp1)] = c;
	    		};
	  	};
	  	replacetemp1[strlen(replacetemp1)+1]='\0';
	
		return(strdup(replacetemp1));
	}
	else
	{
		// return error message
		return(strdup("-- Keine Daten --"));
	}
}


/////////////////////////////////////////////////////////
// Set LCD-Lines on LCD

void activyLCD::SetLCDLines(char text1[21],  char text2[21])
{
  	LCDInit();

  	char text[21];
	char output[30];

	// 1. Line zusammensetzen
  	memset(text,' ',21);
  	snprintf(text,21,"%-20s",text1);

	// Display 1. Line clearen
  	snprintf(output,3, "%s%c", LCD_WRITE_TEXT_LINE0, '\x00');
  	SendCommandToLCD(output,3 );

	// Display 1. Line setzen
  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, text, '\x00');
  	SendCommandToLCD(output,23 );

	// 2. Line zusammensetzen
  	memset(text,' ',21);
  	snprintf(text,21,"%-20s",text2);

	// Display 2. Line clearen
 	snprintf(output,3, "%s%c", LCD_WRITE_TEXT_LINE1, '\x00');
  	SendCommandToLCD(output,3 );

	// Display 2. Line setzen
  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, text,'\x00');
  	SendCommandToLCD(output,23 );

}

/////////////////////////////////////////////////////////
// Set LCD-Line0 on LCD

void activyLCD::SetLCDLine0(char *text1)
{
  	char text[21];
	char output[30];

	// only proceed, if valid data
	if(text1)
	{
	  	LCDInit();
	
		// assemble line 0
	  	memset(text,' ',21);
	  	snprintf(text,21,"%-20s",ReplaceUmlaute(text1));

		// set activy display line 0
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, text, '\x00');
	  	SendCommandToLCD(output,23 );
	}
	else
	{
		// no valid data received
		// set activy display line 0
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, "-- Keine Daten --", '\x00');
	  	SendCommandToLCD(output,23 );
	}
}

/////////////////////////////////////////////////////////
// Set LCD-Line1 on LCD

void activyLCD::SetLCDLine1(char *text1)
{
    	char text[21];
	char output[30];

	// only proceed, if valid data
	if(text1)
	{
		LCDInit();

		// assemble line 1
	  	memset(text,' ',21);
	  	snprintf(text,21,"%-20s",ReplaceUmlaute(text1));

		// set activy display line 1
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, text, '\x00');
	  	SendCommandToLCD(output,23 );
	}
	else
	{
		// no valid data received
		// set activy display line 1
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, "-- Keine Daten --", '\x00');
	  	SendCommandToLCD(output,23 );
	}
}

/////////////////////////////////////////////////////////
// Clear LCD Device

void activyLCD::LCDClear() // clears the lcd display
{
	char output[30];

  	snprintf(output,2, "%s%c", LCD_CLEAR, '\x00');
  	SendCommandToLCD(output,2 );
}

/////////////////////////////////////////////////////////
// Set Power Button

void activyLCD::SetPowerButton(bool status) // switch powerbutton on
{
	char output[30];

  	switch (status) 
	{
    		case 0:
      		snprintf(output,2, "%s", POWERBUTTON_SCANCODE_OFF);
		SendCommandToLCD(output,2 );
      		snprintf(output,2, "%s", POWERBUTTON_OFF);
  		SendCommandToLCD(output,2 );
      		break;

    		case 1:
      		snprintf(output,2, "%s", POWERBUTTON_SCANCODE_ON);
  		SendCommandToLCD(output,2 );
      		snprintf(output,2, "%s", POWERBUTTON_ON);
		SendCommandToLCD(output,2 );
      		break;
  	}
}

/////////////////////////////////////////////////////////
// Set LED Brightness

void activyLCD::SetLEDBrightness(const int brightness)
{
 	int brightness_temp = 0;
	char output[30];

  	switch(brightness) 
	{ 
    		case 0: // superbright
      		brightness_temp = 0;
      		break;

    		case 1: // bright
      		brightness_temp = 50;
      		break;

    		case 2: // semibright
      		brightness_temp = 100;
      		break;

    		case 3: // dark
      		brightness_temp = 150;
      		break;

    		case 4: // off
      		brightness_temp = 255;
      		break;
	}

  	snprintf(output,2, "%s%c", LCD_LEDBRIGHTNESS, brightness_temp);
 	SendCommandToLCD(output,2 );
}

#endif
